# -*- coding: utf-8 -*-
require 'spec_helper'

describe Libmagellan::HTTP do
  let(:lm){ Libmagellan::HTTP.new("http://localhost:3000", consumer_key: "foo", consumer_secret: "bar") }

  describe :request do

    let(:http){ double(:http) }
    let(:res){ double(:res) }

    before do
      allow(Net::HTTP).to receive(:new).with("localhost", 3000).and_return(http)
      allow(http).to receive(:start).and_yield(http)
    end

    describe :get do
      it :success do
        allow(http).to receive(:get).\
          with("/ping", hash_including("Authorization" => an_instance_of(String), "Cache-Control" => "no-store")).\
          and_return(res)
        r = lm.request("/ping")
        expect(r).to eq res
      end

      it "success with client_version" do
        lm.client_version = "0.0.1"
        allow(http).to receive(:get).\
          with("/ping", hash_including("Authorization" => an_instance_of(String), "Cache-Control" => "no-store", "Client-Version" => "0.0.1")).\
          and_return(res)
        r = lm.request("/ping")
        expect(r).to eq res
      end
    end

    describe :post do
      it :success do
        allow(http).to receive(:post).\
          with("/ping", "param1=value1", hash_including("Authorization" => an_instance_of(String), "Cache-Control" => "no-store")).\
          and_return(res)
        r = lm.request("/ping", :post, "param1=value1")
        expect(r).to eq res
      end
      it "accept Hash as body parameter" do
        allow(http).to receive(:post).\
          with("/ping", "param1=value1&param2=value2", hash_including("Authorization" => an_instance_of(String), "Cache-Control" => "no-store")).\
          and_return(res)
        r = lm.request("/ping", :post, {param1: "value1", param2: "value2"})
        expect(r).to eq res
      end
    end

    describe :put do
      it :success do
        allow(http).to receive(:put).\
          with("/ping", "param1=value1&param2=value2", hash_including("Authorization" => an_instance_of(String), "Cache-Control" => "no-store")).\
          and_return(res)
        r = lm.request("/ping", :put, {param1: "value1", param2: "value2"})
        expect(r).to eq res
      end
    end

    describe :delete do
      it :success do
        allow(http).to receive(:delete).\
          with("/ping", hash_including("Authorization" => an_instance_of(String), "Cache-Control" => "no-store")).\
          and_return(res)
        r = lm.request("/ping", :delete)
        expect(r).to eq res
      end
    end
  end

  describe :ping do

    let(:http){ double(:http) }
    let(:res){ double(:res) }

    before do
      allow(Net::HTTP).to receive(:new).with("localhost", 3000).and_return(http)
      allow(http).to receive(:start).and_yield(http)
    end

    it :success do
      allow(res).to receive(:code).and_return("200")
      allow(res).to receive(:body).and_return("pong\n")
      allow(http).to receive(:get).\
        with("/ping", hash_including("Authorization" => an_instance_of(String), "Cache-Control" => "no-store")).\
        and_return(res)
      r = lm.test_ping
      expect(r).to eq lm
    end

    it :connection_failure_1_time do
      allow(http).to receive(:get).\
        with("/ping", hash_including("Authorization" => an_instance_of(String), "Cache-Control" => "no-store")).\
        once.\
        and_raise(Errno::ECONNREFUSED)
      allow(res).to receive(:code).and_return("200")
      allow(res).to receive(:body).and_return("pong\n")
      allow(http).to receive(:get).\
        with("/ping", hash_including("Authorization" => an_instance_of(String), "Cache-Control" => "no-store")).\
        and_return(res)
      r = lm.test_ping
      expect(r).to eq lm
    end

    it :connection_failure do
      allow(http).to receive(:get).\
        with("/ping", hash_including("Authorization" => an_instance_of(String), "Cache-Control" => "no-store")).\
        exactly(3).times.\
        and_raise(Errno::ECONNREFUSED)
      expect{ lm.test_ping(max_retry_count: 2, retry_interval: 0) }.to raise_error(Errno::ECONNREFUSED)
    end

    it :error_response do
      allow(res).to receive(:code).and_return("500")
      allow(res).to receive(:body).and_return("Something wrong!")
      allow(http).to receive(:get).\
        with("/ping", hash_including("Authorization" => an_instance_of(String), "Cache-Control" => "no-store")).\
        and_return(res)
      expect{ lm.test_ping }.to raise_error(Libmagellan::InvalidResponse)
    end
  end
end
